@@ -1,5 +1,11 @@
 /* destest.c */
-/* Copyright (C) 1993 Eric Young - see README for more details */
+/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au).
+ * All rights reserved.
+ * Copyright remains Eric Young's, and as such any Copyright notices in
+ * the code are not to be removed.
+ * See the COPYRIGHT file in the libdes distribution for more details.
+ */
+
 #include <stdio.h>
 #include "des_locl.h" /* for des.h and bcopy macros */
 /* tisk tisk - the test keys don't all have odd parity :-( */
@@ -218,6 +224,8 @@ unsigned char cbc_cksum_data[8]={0x1D,0x26,0x93,0x97,0xf7,0xfe,0x62,0xb4};
 
 char *malloc();
 char *pt();
+void cfb_test();
+void cfb64_test();
 
 main()
 	{
@@ -228,6 +236,7 @@ main()
 	unsigned long cs;
 	unsigned char qret[4][4],cret[8];
 	unsigned long lqret[4];
+	int num;
 	char *str;
 
 	printf("Doing ecb\n");
@@ -305,15 +314,21 @@ main()
 	bcopy(cbc3_iv,iv3b,sizeof(cbc_iv));
 	bzero(cbc_out,40);
 	bzero(cbc_in,40);
+	i=strlen(cbc_data)+1;
+	i=((i+7)/8)*8;
 	des_3cbc_encrypt((C_Block *)cbc_data,(C_Block *)cbc_out,
-		(long)strlen(cbc_data)+1,ks,ks2,(C_Block *)iv3a,
+		16,ks,ks2,(C_Block *)iv3a,
 		(C_Block *)iv3b,DES_ENCRYPT);
-	if (bcmp(cbc_out,cbc3_ok,(strlen(cbc_data)+7)/8*8) != 0)
+	des_3cbc_encrypt((C_Block *)&(cbc_data[16]),(C_Block *)&(cbc_out[16]),
+		(long)i-16,ks,ks2,(C_Block *)iv3a,
+		(C_Block *)iv3b,DES_ENCRYPT);
+	if (bcmp(cbc_out,cbc3_ok,(strlen(cbc_data)+1+7)/8*8) != 0)
 		printf("des_3cbc_encrypt encrypt error\n");
+
 	bcopy(cbc_iv,iv3a,sizeof(cbc_iv));
 	bcopy(cbc3_iv,iv3b,sizeof(cbc_iv));
 	des_3cbc_encrypt((C_Block *)cbc_out,(C_Block *)cbc_in,
-		(long)strlen(cbc_data)+1,ks,ks2,(C_Block *)iv3a,
+		i,ks,ks2,(C_Block *)iv3a,
 		(C_Block *)iv3b,DES_DECRYPT);
 	if (bcmp(cbc_in,cbc_data,strlen(cbc_data)+1) != 0)
 		printf("des_3cbc_encrypt decrypt error\n");
@@ -344,6 +359,9 @@ main()
 	printf("cfb64 ");
 	cfb_test(64,cfb_cipher64);
 
+	printf("cfb64 ");
+	cfb64_test(64,cfb_cipher64);
+
 	bcopy(cfb_iv,cfb_tmp,sizeof(cfb_iv));
 	for (i=0; i<sizeof(plain); i++)
 		des_cfb_encrypt(&(plain[i]),&(cfb_buf1[i]),
@@ -372,6 +390,26 @@ main()
 	if (bcmp(plain,ofb_buf2,sizeof(ofb_buf2)) != 0)
 		printf("ofb_encrypt decrypt error\n");
 
+	printf("Doing ofb64\n");
+	key_sched((C_Block *)ofb_key,ks);
+	bcopy(ofb_iv,ofb_tmp,sizeof(ofb_iv));
+	bzero(ofb_buf1,sizeof(ofb_buf1));
+	bzero(ofb_buf2,sizeof(ofb_buf1));
+	num=0;
+	for (i=0; i<sizeof(plain); i++)
+		{
+		des_ofb64_encrypt(&(plain[i]),&(ofb_buf1[i]),1,ks,
+			(C_Block *)ofb_tmp,&num);
+		}
+	if (bcmp(ofb_cipher,ofb_buf1,sizeof(ofb_buf1)) != 0)
+		printf("ofb64_encrypt encrypt error\n");
+	bcopy(ofb_iv,ofb_tmp,sizeof(ofb_iv));
+	num=0;
+	des_ofb64_encrypt(ofb_buf1,ofb_buf2,(long)sizeof(ofb_buf1),ks,
+		(C_Block *)ofb_tmp,&num);
+	if (bcmp(plain,ofb_buf2,sizeof(ofb_buf2)) != 0)
+		printf("ofb64_encrypt decrypt error\n");
+
 	printf("Doing cbc_cksum\n");
 	key_sched((C_Block *)cbc_key,ks);
 	cs=cbc_cksum((C_Block *)cbc_data,(C_Block *)cret,
@@ -468,7 +506,7 @@ unsigned char *p;
 	return(ret);
 	}
 	
-int cfb_test(bits,cfb_cipher)
+void cfb_test(bits,cfb_cipher)
 int bits;
 char *cfb_cipher;
 	{
@@ -495,3 +533,39 @@ char *cfb_cipher;
 			printf("%s\n",pt(&(cfb_buf1[i])));
 		}
 	}
+
+void cfb64_test(bits,cfb_cipher)
+int bits;
+char *cfb_cipher;
+	{
+	des_key_schedule ks;
+	int i,n;
+
+	key_sched((C_Block *)cfb_key,ks);
+	bcopy(cfb_iv,cfb_tmp,sizeof(cfb_iv));
+	n=0;
+	des_cfb64_encrypt(plain,cfb_buf1,(long)12,ks,
+		(C_Block *)cfb_tmp,&n,DES_ENCRYPT);
+	des_cfb64_encrypt(&(plain[12]),&(cfb_buf1[12]),
+		(long)sizeof(plain)-12,ks,
+		(C_Block *)cfb_tmp,&n,DES_ENCRYPT);
+	if (bcmp(cfb_cipher,cfb_buf1,sizeof(plain)) != 0)
+		{
+		printf("cfb_encrypt encrypt error\n");
+		for (i=0; i<24; i+=8)
+			printf("%s\n",pt(&(cfb_buf1[i])));
+		}
+	bcopy(cfb_iv,cfb_tmp,sizeof(cfb_iv));
+	n=0;
+	des_cfb64_encrypt(cfb_buf1,cfb_buf2,(long)17,ks,
+		(C_Block *)cfb_tmp,&n,DES_DECRYPT);
+	des_cfb64_encrypt(&(cfb_buf1[17]),&(cfb_buf2[17]),
+		(long)sizeof(plain)-17,ks,
+		(C_Block *)cfb_tmp,&n,DES_DECRYPT);
+	if (bcmp(plain,cfb_buf2,sizeof(plain)) != 0)
+		{
+		printf("cfb_encrypt decrypt error\n");
+		for (i=0; i<24; i+=8)
+			printf("%s\n",pt(&(cfb_buf1[i])));
+		}
+	}
