@@ -1,5 +1,11 @@
 /* rpc_enc.c */
-/* Copyright (C) 1994 Eric Young - see README for more details */
+/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au).
+ * All rights reserved.
+ * Copyright remains Eric Young's, and as such any Copyright notices in
+ * the code are not to be removed.
+ * See the COPYRIGHT file in the libdes distribution for more details.
+ */
+
 #include "rpc_des.h"
 #include "des_locl.h"
 #include "version.h"
@@ -29,8 +35,9 @@ struct desparams *desp;
 		 * in secure_rpc.
 		 * Libdes's cbc encrypt does not copy back the iv,
 		 * so we have to do it here. */
-		a=&(desp->UDES.UDES_buf[len-8]);
-		b=&(desp->des_ivec[0]);
+		a=(char *)&(desp->UDES.UDES_buf[len-8]);
+		b=(char *)&(desp->des_ivec[0]);
+
 		*(a++)= *(b++); *(a++)= *(b++);
 		*(a++)= *(b++); *(a++)= *(b++);
 		*(a++)= *(b++); *(a++)= *(b++);
